@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *    GDMflexiserver - run a flexible server
  *    (c)2001 Queen of England
  *    
@@ -19,10 +20,6 @@
  */
 
 #include "config.h"
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
-#include <gdk/gdkx.h>
-#include <X11/Xauth.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,20 +33,29 @@
 #include <errno.h>
 #include <pwd.h>
 
+#include <glib/gi18n.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <libgnome/libgnome.h> /* for gnome_config */
+#include <libgnomeui/libgnomeui.h> /* for gnome_program */
+
 #include <viciousui.h>
 
 #include "gdm.h"
 #include "gdmcomm.h"
+#include "gdmcommon.h"
 
 static GSList *xservers = NULL;
 static gboolean got_standard = FALSE;
 static gboolean use_xnest = FALSE;
 static gboolean authenticate = FALSE;
 static gboolean no_lock = FALSE;
+static gboolean monte_carlo_pi = FALSE;
 static const char *send_command = NULL;
 static const char *server = NULL;
 static const char *chosen_server = NULL;
 static gboolean debug = FALSE;
+static gboolean startnew = FALSE;
 static char *auth_cookie = NULL;
 
 static int
@@ -294,6 +300,60 @@ row_activated (GtkTreeView *view, GtkTreePath *path, GtkTreeViewColumn *col, gpo
 	gtk_dialog_response (GTK_DIALOG (dialog), RESPONSE_OPEN_EXISTING_DISPLAY);
 }
 
+static gboolean
+is_program_in_path (const char *program)
+{
+	char *tmp = g_find_program_in_path (program);
+	if (tmp != NULL) {
+		g_free (tmp);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+static void
+maybe_lock_screen (void)
+{
+	gboolean   use_gscreensaver = FALSE;
+	GError    *error            = NULL;
+	char      *command;
+	GdkScreen *screen;
+
+	if (is_program_in_path ("gnome-screensaver-command"))
+		use_gscreensaver = TRUE;
+	else if (! is_program_in_path ("xscreensaver-command"))
+		return;
+
+	if (use_gscreensaver) {
+		command = g_strdup ("gnome-screensaver-command --lock");
+	} else {
+		command = g_strdup ("xscreensaver-command -lock");
+	}
+
+	screen = gdk_screen_get_default ();
+
+	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+		g_warning (_("Can't lock screen"));
+		g_error_free (error);
+	}
+
+	g_free (command);
+
+	if (use_gscreensaver) {
+		command = g_strdup ("gnome-screensaver-command --throttle");
+	} else {
+		command = g_strdup ("xscreensaver-command -throttle");
+	}
+
+	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+		g_warning (_("Can't disable xscreensaver display hacks"));
+		g_error_free (error);
+	}
+
+	g_free (command);
+}
+
 static void
 run_logged_in_dialogue (char **vec)
 {
@@ -306,6 +366,11 @@ run_logged_in_dialogue (char **vec)
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
 
+        if (startnew == TRUE) {
+                /* Just return if the user doesn't want to see the dialog */
+                return;
+	}
+
 	dialog = gtk_dialog_new_with_buttons (_("Open Displays"),
 					      NULL /* parent */,
 					      0 /* flags */,
@@ -384,12 +449,7 @@ run_again:
 			/* we switched to a different screen as a result of this,
 			 * lock the current screen */
 			if ( ! no_lock && vt != get_cur_vt () && vt >= 0) {
-				char *argv[3] = {"xscreensaver-command", "-lock", NULL};
-				if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-					g_warning (_("Can't lock screen"));
-				argv[1] = "-throttle";
-				if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-					g_warning (_("Can't disable xscreensaver display hacks"));
+				maybe_lock_screen ();
 			}
 
 			change_vt (vt);
@@ -469,13 +529,15 @@ check_for_users (void)
 }
 
 static void
-read_servers (void)
+read_servers (gchar *config_file)
 {
 	gpointer iter;
+	gchar *config_sections;
 	char *k;
 
 	/* Find server definitions */
-	iter = gnome_config_init_iterator_sections ("=" GDM_CONFIG_FILE "=/");
+	config_sections = g_strdup_printf ("=%s=/", config_file);
+	iter = gnome_config_init_iterator_sections (config_sections);
 	iter = gnome_config_iterator_next (iter, &k, NULL);
 
 	while (iter) {
@@ -483,7 +545,7 @@ read_servers (void)
 			char *section;
 			GdmXServer *svr;
 
-			section = g_strdup_printf ("=" GDM_CONFIG_FILE "=/%s/", k);
+			section = g_strdup_printf ("=%s=/", config_file);
 			gnome_config_push_prefix (section);
 
 			if ( ! gnome_config_get_bool
@@ -608,12 +670,32 @@ choose_server (void)
 	return NULL;
 }
 
+static void
+calc_pi (void)
+{
+	unsigned long n = 0, h = 0;
+	double x, y;
+	printf ("\n");
+	for (;;) {
+		x = g_random_double ();
+		y = g_random_double ();
+		if (x*x + y*y <= 1)
+			h++;
+		n++;
+		if ( ! (n & 0xfff))
+			printf ("pi ~~ %1.10f\t(%lu/%lu * 4) iteration: %lu \r",
+				((double)h)/(double)n * 4.0, h, n, n);
+	}
+}
+
 struct poptOption options [] = {
 	{ "command", 'c', POPT_ARG_STRING, &send_command, 0, N_("Send the specified protocol command to gdm"), N_("COMMAND") },
 	{ "xnest", 'n', POPT_ARG_NONE, &use_xnest, 0, N_("Xnest mode"), NULL },
 	{ "no-lock", 'l', POPT_ARG_NONE, &no_lock, 0, N_("Do not lock current screen"), NULL },
 	{ "debug", 'd', POPT_ARG_NONE, &debug, 0, N_("Debugging output"), NULL },
 	{ "authenticate", 'a', POPT_ARG_NONE, &authenticate, 0, N_("Authenticate before running --command"), NULL },
+        { "startnew", 's', POPT_ARG_NONE, &startnew, 0, N_("Start new flexible session; do not show popup"), NULL },
+	{ "monte-carlo-pi", 0, POPT_ARG_NONE, &monte_carlo_pi, 0, NULL, NULL },
 	POPT_AUTOHELP
 	{ NULL, 0, 0, NULL, 0}
 };
@@ -623,6 +705,7 @@ int
 main (int argc, char *argv[])
 {
 	GtkWidget *dialog;
+	gchar *config_file;
 	char *command;
 	char *version;
 	char *ret;
@@ -644,14 +727,27 @@ main (int argc, char *argv[])
 		      GNOME_PARAM_POPT_CONTEXT, &ctx,
 		      NULL);	
 
+	if (monte_carlo_pi) {
+		calc_pi ();
+		return 0;
+	}
+
 	gdmcomm_set_debug (debug);
 
 	args = poptGetArgs (ctx);
 	if (args != NULL && args[0] != NULL)
 		server = args[0];
 
-	if ( ! gdmcomm_check (TRUE /* gui_bitching */))
+	config_file = gdm_common_get_config_file ();
+	if (config_file == NULL) {
+		g_print (_("Could not access GDM configuration file.\n"));
+		exit (0);
+	}
+
+	if ( ! gdmcomm_check (config_file, TRUE /* gui_bitching */)) {
+		g_free (config_file);
 		return 1;
+	}
 
 	if (send_command != NULL) {
 		if (authenticate)
@@ -660,6 +756,7 @@ main (int argc, char *argv[])
 					"2.2.4.0", 5);
 		if (ret != NULL) {
 			g_print ("%s\n", ret);
+			g_free (config_file);
 			return 0;
 		} else {
 			dialog = ve_hig_dialog_new
@@ -676,6 +773,7 @@ main (int argc, char *argv[])
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
+			g_free (config_file);
 			return 1;
 		}
 	}
@@ -706,6 +804,7 @@ main (int argc, char *argv[])
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
+			g_free (config_file);
 			return 1;
 		}
 		command = g_strdup_printf (GDM_SUP_FLEXI_XNEST " %s %d %s %s",
@@ -734,10 +833,11 @@ main (int argc, char *argv[])
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
+			g_free (config_file);
 			return 1;
 		}
 
-		read_servers ();
+		read_servers (config_file);
 		server = choose_server ();
 		if (server == NULL)
 			command = g_strdup (GDM_SUP_FLEXI_XSERVER);
@@ -747,19 +847,18 @@ main (int argc, char *argv[])
 		version = "2.2.4.0";
 	}
 
+	g_free (config_file);
+
 	ret = gdmcomm_call_gdm (command, auth_cookie, version, 5);
+	g_free (command);
+
 	if (ret != NULL &&
 	    strncmp (ret, "OK ", 3) == 0) {
 
 		/* if we switched to a different screen as a result of this,
 		 * lock the current screen */
 		if ( ! no_lock && ! use_xnest) {
-			char *argv[3] = {"xscreensaver-command", "-lock", NULL};
-			if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-				g_warning (_("Can't lock screen"));
-			argv[1] = "-throttle";
-			if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-				g_warning (_("Can't disable xscreensaver display hacks"));
+				maybe_lock_screen ();
 		}
 
 		/* all fine and dandy */
