@@ -198,7 +198,7 @@ install_response (GtkWidget *file_dialog,
 	gtk_widget_destroy (file_dialog);
 }
 
-static GtkWidget *
+void
 add_preview_widget (GtkWidget *widget)
 {
 	GtkWidget *vbox;
@@ -237,6 +237,7 @@ browse_button_cb (GtkWidget *widget, gpointer data)
 						   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						   GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 						   NULL);
+	gtk_file_chooser_set_show_hidden (GTK_FILE_CHOOSER (file_dialog), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_dialog), TRUE);
 
 	if (facedir && g_file_test (facedir, G_FILE_TEST_IS_DIR)) {
