@@ -1,3 +1,21 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <math.h>
@@ -8,12 +26,14 @@
 #include "vicious.h"
 
 #include "gdm.h"
+#include "gdmcommon.h"
 #include "greeter.h"
 #include "greeter_item.h"
 #include "greeter_events.h"
 #include "greeter_system.h"
 #include "greeter_canvas_item.h"
 #include "greeter_configuration.h"
+#include "greeter_canvas_text.h"
 
 static void
 apply_tint (GdkPixbuf *pixbuf, guint32 tint_color)
@@ -244,9 +264,7 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 	      item->data.pixmap.pixbufs[i] = g_object_ref (item->data.pixmap.pixbufs[0]);
 	    else
 	      item->data.pixmap.pixbufs[i] =
-	        rsvg_pixbuf_from_file_at_size (item->data.pixmap.files[i],
-					       rect.width, rect.height,
-					       NULL);
+                  gdk_pixbuf_new_from_file_at_size (item->data.pixmap.files[i], rect.width, rect.height, NULL);
 	  }
 	else
 	  item->data.pixmap.pixbufs[i] = NULL;
@@ -294,7 +312,7 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 	    just = GTK_JUSTIFY_CENTER;
 
     item->item = gnome_canvas_item_new (group,
-					GNOME_TYPE_CANVAS_TEXT,
+					GREETER_TYPE_CANVAS_TEXT,
 					"text", "",
 					"x", x1,
 					"y", y1,
@@ -343,8 +361,11 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
     
   case GREETER_ITEM_TYPE_ENTRY:
     entry = gtk_entry_new ();
+    gtk_widget_set_name (entry, "user-pw-entry");
     gtk_entry_set_has_frame (GTK_ENTRY (entry), FALSE);
-    if (GdmUseCirclesInEntry)
+    if (GdmUseInvisibleInEntry)
+      gtk_entry_set_invisible_char (GTK_ENTRY (entry), 0);
+    else if (GdmUseCirclesInEntry)
       gtk_entry_set_invisible_char (GTK_ENTRY (entry), 0x25cf);
     gtk_widget_modify_font (entry, item->data.text.fonts[GREETER_ITEM_STATE_NORMAL]);
 
@@ -396,7 +417,7 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
      * to it then, depending on the type.  Likely userlist is the
      * only type we support */
     list = gtk_tree_view_new ();
-    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list), TRUE);
+
     swin = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin),
     					GTK_SHADOW_NONE);
@@ -533,7 +554,7 @@ greeter_canvas_item_break_set_string (GreeterItemInfo *info,
 		canvas_item = real_item;
 	else
 		canvas_item = gnome_canvas_item_new (gnome_canvas_root (canvas),
-						     GNOME_TYPE_CANVAS_TEXT,
+						     GREETER_TYPE_CANVAS_TEXT,
 						     textattr, "",
 						     "x", 0.0,
 						     "y", 0.0,
